#ifndef DESIGN_H
#define DESIGN_H

#include <iostream>
#include <conio.h>
#include <fstream>
#include <windows.h>
using namespace std;

// costanti con le dimensioni della classica schermata DOS
const int nROW = 24,    // numero di righe
          nCOLUMN = 79; // numero di colonne

// costanti coi valori dei tasti che l'utente pu premere
const char UP = 119,             // W = s
           DOWN = 115,           // S = gi
           LEFT = 97,            // A = sinistra
           RIGHT = 100,          // D = destra
           EXIT = 27,            // Esc = esci
           DRAW = 49,            // 1 = attiva/disattiva matita
           CANCEL = 50,          // 2 = attiva/disattiva gomma
           WRITE = 51,           // 3 = attiva/disattiva modalit scrittura
           COLOR_UP = 105,       // I = cambia colore carattere/avanti
           COLOR_DOWN = 107,     // K = cambia colore carattere/indietro
           BGCOLOR_UP = 108,     // L = cambia colore sfondo/avanti
           BGCOLOR_DOWN = 106,   // J = cambia colore sfondo/indietro
           OBJECT_LEFT = 113,    // Q = cambia carattere/avanti
           OBJECT_RIGHT = 101,   // E = cambia carattere/indietro
           SAVE = 48,            // 0 = salva disegno
           CLEAR = 32;           // Space = cancela disegno

                               // rettangoli
const char OBJECT_TOOLS[29] = {35, 176, 177, 178, 219, 220, 223,
                               // pezzi di tabella
                               218, 196, 194, 191, 179, 195, 197, 180, 192, 193, 217,
                               // pezzi di tabella doppio spessore
                               201, 205, 203, 187, 186, 204, 206, 185, 200, 202, 188};
                                    
// struttura che contiene le inforamazioni sui, per cos dire, pixel del disegno
struct pixel
{
       unsigned char character;
       short color;
};     

/* procedura per spostare il cursore sullo schermo, il primo valore lo sposta
orizzontalmente, il secondo verticalmente */
void gotoxy(unsigned short y, unsigned short x)
{
    COORD coor = {x, y};
    HANDLE hOutput;
    hOutput = GetStdHandle(STD_OUTPUT_HANDLE);
    SetConsoleCursorPosition(hOutput,coor);
}

/* procedura per cambiare il colore dei caratteri */
void SelectColor (unsigned short color)
{
     HANDLE hCon = GetStdHandle (STD_OUTPUT_HANDLE);
     SetConsoleTextAttribute (hCon, color);
} 

// classe per la gestione dei disegni
class design
{
      // matrice in cui viene salvato l'intero disegno
      pixel screen_view[nROW][nCOLUMN];
      public:
             void clear();
             design() { clear(); }
             void draw();
             void save();
             void load(string);
             };

// procedura per pulire la matrice
void design::clear()
{
     int i, j;
     for (i=0; i<nROW; i++) {
         for (j=0; j<nCOLUMN; j++) {
             screen_view[i][j].character = ' ';
             screen_view[i][j].color = 7;
             }
         }
}

// funzione che restituisce vero se il tasto premuto  uno di quelli a
// disposizione dell'utente
bool check_keystate(char move)
{
     if (move!=UP && move!=DOWN && move!=LEFT && move!=RIGHT) {
        if (move!=DRAW && move!=CANCEL && move!=CLEAR && move!=WRITE) {
           if (move!=COLOR_UP && move!=COLOR_DOWN && move!=BGCOLOR_UP && move!=BGCOLOR_DOWN) {
              if (move!=EXIT && move!=SAVE && move!=OBJECT_RIGHT && move!=OBJECT_LEFT) return true;
              }
           }
        }
     return false;
}

// procedura per disegnare
void design::draw()
{     
     int mx,        // movimento in orizzontale
         my,        // movimento in verticale
         x = 12,    // posizione del cursore in verticale
         y = 40;    // posizione del cursore in orizzontale
     
     short color = 15,     // colore del puntatore
           bgcolor = 0,    // colore di sfondo
           object_num = 0; // tipo di carattere
     
     // gli interruttori per disegnare, cancellare e colorare
     bool draw_switch = false,
          cancel_switch = false,
          color_switch = false;
         
     char button;
     
     for (;;) {
         // l'utente preme un tasto finch non ne preme uno di quelli precedentemente indicati
         do {
             button = getch();
             } while(check_keystate(button));
     
     // in base al tasto premuto viene eseguita un'istruzione diversa
     switch (button) {
            case UP: mx=-1; my=0; break;
            case DOWN: mx=1; my=0; break;
            case LEFT: mx=0; my=-1; break;
            case RIGHT: mx=0; my=1; break;
            case DRAW:
                 mx=0; my=0;
                 if(draw_switch) draw_switch = false;
                 else draw_switch = true;
                 cancel_switch = false;
                 break;
            case CANCEL:
                 mx=0; my=0;
                 if(cancel_switch) cancel_switch = false;
                 else cancel_switch = true;
                 draw_switch = false;
                 break;
            case WRITE:
                 mx=0; my=0;
                 button = getch();
                 while (button!=13 && button!=8) {
                       cout << button;
                       screen_view[x][y].character = button;
                       screen_view[x][y].color = bgcolor*16 + color;
                       if (y == nCOLUMN) {
                          y = 0;
                          if (x == nROW-1) {
                             button = 13;
                             x--;
                             }
                          else x++;
                          }
                       else y++;
                       button = getch();
                       }
                 break;
            case COLOR_UP:
                 mx=0; my=0;
                 if(color == 15) color = 0;
                 else color++;
                 break;
            case COLOR_DOWN:
                 mx=0; my=0;
                 if(color == 0) color = 15;
                 else color--;
                 break;
            case BGCOLOR_UP:
                 mx=0; my=0;
                 if(bgcolor == 15) bgcolor = 0;
                 else bgcolor++;
                 break;
            case BGCOLOR_DOWN:
                 mx=0; my=0;
                 if(bgcolor == 0) bgcolor = 15;
                 else bgcolor--;
                 break;
            case OBJECT_LEFT:
                 mx=0; my=0;
                 if(object_num == 0) object_num = 27;
                 else object_num--;
                 break;
            case OBJECT_RIGHT:
                 mx=0; my=0;
                 if(object_num == 27) object_num = 0;
                 else object_num++;
                 break;
            case CLEAR:
                 mx=0; my=0;
                 system("cls");
                 clear();
                 break;
            case SAVE:
                 mx=0; my=0;
                 save();
                 break;
            }
     
     if(button == EXIT) break;
     
     // aggiorna le coordinate del cursore coi rispettivi controlli che il cursore
     // non finisca fuori dalla finestra 
     x += mx;
     if(x == nROW) x--;
     if(x == -1) x++;
     
     y += my;
     if(y == nCOLUMN) y--;
     if(y == -1) y++;
     gotoxy(x, y);
     SelectColor(bgcolor*16 + color);
     
     // in base all'interruttore attivo esegue uno dei seguenti effetti
     if (draw_switch) {
        cout << OBJECT_TOOLS[object_num];
        screen_view[x][y].character = OBJECT_TOOLS[object_num];
        screen_view[x][y].color = bgcolor*16 + color;
        }
        
     if (cancel_switch) {
        SelectColor(0);
        cout <<' ';
        screen_view[x][y].character = ' ';
        screen_view[x][y].color = 0;
        }
     }
     // pulisce la matrice alla fine dell'esecuzione
     clear();
}

// precedura per salvare il disegno
void design::save()
{
     system("color 0F");
     string fname, file_name;
     char ok;
     
     system("cls");
     // chiede il nome dell'immagine da salvare
     cout <<"\n\n\nInserire nome dell'immagine da salvare: ";
     cin >> fname;
     
     // aggiunge al nome immesso l'estensione e l'indirizio
     file_name = "../Pics/" + fname + ".dat";
     // dichiaro la variabile di stream
     ofstream fout;
     // crea ed apre il file con il nome che si  appena inserito specificando
     // che sar un flusso di output dal programma al file (ios::out) e che 
     // il file sar di tipo binario (ios__binary)
     fout.open(file_name.c_str(), ios::out | ios::binary);
     // scrivo sul file la matrice sotto forma di classe
     fout.write((char*)this, sizeof(*this));
     // chiudo il file
     fout.close();
     
     // chiedo se si vuole continuare il disegno o no
     cout <<"\n\nVuole tornare al disegno (s=si', n=no): ";
     do { ok = getch(); } while(ok!=115 && ok!=110);
     if(ok == 115) load("file_name");
     else exit(0);
}

void design::load(string file_name)
{
     int i, j;
     
     // dichiaro la variabile di stream
     ifstream fin;
     // crea ed apre il file con il nome che si  appena inserito specificando
     // che sar un flusso di input dal file al programma (ios::in) e che 
     // il file sar di tipo binario (ios__binary)
     fin.open(file_name.c_str(), ios::in | ios::binary);
     // se l'apertura non avviene segnalo l'errore
     if (!fin) {
        cerr <<"\n\nerrore nell'apertura dell'immagine, possibili cause:\n";
        cerr <<"\n - il nome dell'immagine e' errato\n";
        cerr <<"\n - l'immagine non e' presente nella cartella delle immagini\n";
        getch();
        }
     else {
          system("cls");
          // legge le informazioni contenute nel file sotto forma di classe design
          fin.read((char*)this, sizeof(*this));
          // stampo a video le informazioni appena ricevute
          for (i=0; i<nROW; i++) {
              for (j=0; j<nCOLUMN; j++) {
                  SelectColor(screen_view[i][j].color);
                  cout << screen_view[i][j].character;
                  }
              cout << endl;
              }
          // chiudo il file
          fin.close();
          // parte la procedura per disegnare avendo come base il disegno appena aperto
          draw();
          }
}

#endif
